#include "widget.h"

#include <QGroupBox>

Widget::Widget(QWidget *parent) :
	QWidget(parent)
{
	mMainLayout = new QGridLayout();

	mCodeTree = new QTreeWidget();
	mCodeTree->setHeaderLabels(QStringList() << "Numer" << "Instrukcja" << "Argumenty");
	mCodeTree->addTopLevelItem(new InstructionItem(InstructionItem::INST_H));
	updateInstructionNumbers();
	connect(mCodeTree, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), this,
		SLOT(handleInstructionChange(QTreeWidgetItem*,QTreeWidgetItem*)));

	mMainLayout->addWidget(mCodeTree, 0, 0, 6, 3);

	mAddInstruction = new QPushButton(QString::fromUtf8("Dodaj"));
	connect(mAddInstruction, SIGNAL(clicked()), this, SLOT(addInstruction()));
	mDeleteInstruction = new QPushButton(QString::fromUtf8("Usuń"));
	connect(mDeleteInstruction, SIGNAL(clicked()), this, SLOT(deleteInstruction()));
	mAvaliableInstructions = new QComboBox();
	mAvaliableInstructions->insertItem(0, QString::fromUtf8("IDŹ (G)"),
		QVariant((int)InstructionItem::INST_G));
	mAvaliableInstructions->insertItem(1, QString::fromUtf8("CZEKAJ 1[s] (W)"),
		QVariant((int)InstructionItem::INST_W1S));
	mAvaliableInstructions->insertItem(2, QString::fromUtf8("CZEKAJ 2[s] (W)"),
		QVariant((int)InstructionItem::INST_W2S));
	mAvaliableInstructions->insertItem(3, QString::fromUtf8("CZEKAJ 5[s] (W)"),
		QVariant((int)InstructionItem::INST_W5S));
	mAvaliableInstructions->insertItem(4, QString::fromUtf8("CZEKAJ 10[s] (W)"),
		QVariant((int)InstructionItem::INST_W10S));
	mAvaliableInstructions->insertItem(5, QString::fromUtf8("CZEKAJ 20[s] (W)"),
		QVariant((int)InstructionItem::INST_W20S));
	mAvaliableInstructions->insertItem(6, QString::fromUtf8("STOP (S)"),
		QVariant((int)InstructionItem::INST_S));
	mAvaliableInstructions->insertItem(7, QString::fromUtf8("DOM (H)"),
		QVariant((int)InstructionItem::INST_H));

	mMainLayout->addWidget(mAddInstruction, 6, 0, 1, 1);
	mMainLayout->addWidget(mDeleteInstruction, 6, 1, 1, 1);
	mMainLayout->addWidget(mAvaliableInstructions, 6, 2, 1, 1);

	for (int i = 0; i < 6; i++) {
		mServoValue[i] = 0;

		if (i == 0) {
			mServoValue[i] = 90;
		} else if (i == 1) {
			mServoValue[i] = 45;
		} else if (i == 2 || i == 3) {
			mServoValue[i] = 180;
		} else if (i == 4) {
			mServoValue[i] = 90;
		} else if (i == 5) {
			mServoValue[i] = 10;
		}

		QGridLayout *gridLayout = new QGridLayout();
		QGroupBox *groupBox = new QGroupBox(QString("Serwo %1").arg(i));
		groupBox->setLayout(gridLayout);

		mServoValueLabel[i] = new QLabel(QString("%1").arg(mServoValue[i]) + QString::fromUtf8("\xc2\xb0"));
		mServoValueLabel[i]->setAlignment(Qt::AlignCenter);

		mIncButton[i] = new ValueChangeButton();
		mIncButton[i]->setText(">");
		mIncButton[i]->setId(i);
		mIncButton[i]->setDelta(1);
		connect(mIncButton[i], SIGNAL(buttonClicked(int,double)), this, SLOT(changeServoValue(int,double)));

		mIncFastButton[i] = new ValueChangeButton();
		mIncFastButton[i]->setText(">>");
		mIncFastButton[i]->setId(i);
		mIncFastButton[i]->setDelta(5);
		connect(mIncFastButton[i], SIGNAL(buttonClicked(int,double)), this, SLOT(changeServoValue(int,double)));

		mDecButton[i] = new ValueChangeButton();
		mDecButton[i]->setText("<");
		mDecButton[i]->setId(i);
		mDecButton[i]->setDelta(-1);
		connect(mDecButton[i], SIGNAL(buttonClicked(int,double)), this, SLOT(changeServoValue(int,double)));

		mDecFastButton[i] = new ValueChangeButton();
		mDecFastButton[i]->setText("<<");
		mDecFastButton[i]->setId(i);
		mDecFastButton[i]->setDelta(-5);
		connect(mDecFastButton[i], SIGNAL(buttonClicked(int,double)), this, SLOT(changeServoValue(int,double)));

		mValueSlider[i] = new ValueChangeSlider();
		if (i == 0 || i == 2 || i == 3 || i == 4) {
			mValueSlider[i]->setMinimum(0);
			mValueSlider[i]->setMaximum(180);
		} else if (i == 1) {
			mValueSlider[i]->setMinimum(15);
			mValueSlider[i]->setMaximum(165);
		} else if (i == 5) {
			mValueSlider[i]->setMinimum(10);
			mValueSlider[i]->setMaximum(73);
		}
		mValueSlider[i]->setSingleStep(1);
		mValueSlider[i]->setOrientation(Qt::Horizontal);
		mValueSlider[i]->setValue((int)mServoValue[i]);
		mValueSlider[i]->setId(i);
		connect(mValueSlider[i], SIGNAL(sliderValueChanged(int,double)), this, SLOT(updateServoValue(int,double)));

		int column = 0;
		gridLayout->addWidget(mDecFastButton[i], 0, column++);
		gridLayout->addWidget(mDecButton[i], 0, column++);
		gridLayout->addWidget(mServoValueLabel[i], 0, column++);
		gridLayout->addWidget(mIncButton[i], 0, column++);
		gridLayout->addWidget(mIncFastButton[i], 0, column++);
		gridLayout->addWidget(mValueSlider[i], 1, 0, 1, column);

		mMainLayout->addWidget(groupBox, i, 3);
	}

	setLayout(mMainLayout);
}

Widget::~Widget()
{
}

QString Widget::servoValuesToArguments()
{
	QString arguments;

	for (int i = 0; i < 5; i++) {
		arguments += QString("%1,").arg(mServoValue[i]);
	}
	arguments += QString("%1").arg(mServoValue[5]);

	return arguments;
}

void Widget::handleInstructionChange(QTreeWidgetItem *current, QTreeWidgetItem *)
{
	if (current == 0) {
		return;
	}

	if (current->type() == InstructionItem::INST_G) {
		QStringList argList = ((InstructionItem*)current)->arguments().split(',');
		for (int i = 0; i < 6; i++) {
			updateServoValue(i, argList[i].toDouble());
		}
	}
}

void Widget::updateInstructionNumbers()
{
	for (int i = 0; i < mCodeTree->topLevelItemCount(); i++) {
		((InstructionItem*)mCodeTree->topLevelItem(i))->setNumber(i+1);
	}
}

void Widget::changeServoValue(int servoNumber, double delta)
{
	double value = mServoValue[servoNumber] + delta;

	if (value < mValueSlider[servoNumber]->minimum()) {
		value = mValueSlider[servoNumber]->minimum();
	} else if (value > mValueSlider[servoNumber]->maximum()) {
		value = mValueSlider[servoNumber]->maximum();
	}

	updateServoValue(servoNumber, value);
}

void Widget::updateServoValue(int servoNumber, double value)
{
	mServoValue[servoNumber] = value;
	mServoValueLabel[servoNumber]->setText(QString("%1").arg(mServoValue[servoNumber]) + QString::fromUtf8("\xc2\xb0"));
	mValueSlider[servoNumber]->setValue((int)mServoValue[servoNumber]);

	if (mCodeTree->currentItem() == 0 || mCodeTree->currentItem()->type() != InstructionItem::INST_G) {
		return;
	}

	((InstructionItem*)mCodeTree->currentItem())->setArguments(servoValuesToArguments());
}

void Widget::addInstruction()
{
	int currentIndex = mCodeTree->indexOfTopLevelItem(mCodeTree->currentItem());

	InstructionItem::InstructionType type =
		(InstructionItem::InstructionType)mAvaliableInstructions->itemData(mAvaliableInstructions->currentIndex()).toInt();

	InstructionItem *itemToCopy = 0;

	if (type == InstructionItem::INST_G) {
		for (int i = currentIndex; i > -1; i--) {
			if (mCodeTree->topLevelItem(i)->type() == InstructionItem::INST_G) {
				itemToCopy = (InstructionItem*)mCodeTree->topLevelItem(i);
				break;
			}
		}
	}

	InstructionItem *item;

	if (itemToCopy != 0) {
		item = new InstructionItem(*itemToCopy);
	} else {
		item = new InstructionItem(type);
	}

	if (currentIndex != -1) {
		mCodeTree->insertTopLevelItem(currentIndex+1, item);
	} else {
		mCodeTree->addTopLevelItem(item);
	}

	updateInstructionNumbers();
}

void Widget::deleteInstruction()
{
	int currentIndex = mCodeTree->indexOfTopLevelItem(mCodeTree->currentItem());
	delete mCodeTree->takeTopLevelItem(currentIndex);
	updateInstructionNumbers();
}
