#ifndef WIDGET_H
#define WIDGET_H

#include <QWidget>
#include <QGridLayout>
#include <QLabel>
#include <QPushButton>
#include <QTreeWidget>
#include <QComboBox>

#include "valuechangebutton.h"
#include "valuechangeslider.h"

#include "instructionitem.h"

class Widget : public QWidget
{
	Q_OBJECT

public:
	explicit Widget(QWidget *parent = 0);
	~Widget();

	QString servoValuesToArguments();

public slots:
	void handleInstructionChange(QTreeWidgetItem *current, QTreeWidgetItem *);
	void updateInstructionNumbers();
	void changeServoValue(int servoNumber, double delta);
	void updateServoValue(int servoNumber, double value);

	void addInstruction();
	void deleteInstruction();

private:
	double mServoValue[6];

	QGridLayout *mMainLayout;

	QTreeWidget *mCodeTree;
	QPushButton *mAddInstruction;
	QPushButton *mDeleteInstruction;
	QComboBox *mAvaliableInstructions;

	QLabel *mServoValueLabel[6];
	ValueChangeButton *mIncButton[6];
	ValueChangeButton *mIncFastButton[6];
	ValueChangeButton *mDecButton[6];
	ValueChangeButton *mDecFastButton[6];
	ValueChangeSlider *mValueSlider[6];

};

#endif // WIDGET_H
